@@ -12,7 +12,7 @@ from ubersmith.exceptions import (
     ResponseError,
     UpdatingTokenResponse,
 )
-from ubersmith.utils import append_qs, urlencode_unicode
+from ubersmith.utils import append_qs, urlencode_unicode, to_nested_php_args
 
 __all__ = [
     'METHODS',
@@ -29,6 +29,8 @@ METHODS = {
     u'client.ach_delete': u'Delete a Bank Account',
     u'client.ach_update': u'Update a Bank Account',
     u'client.add': u'Add a New Client',
+    u'client.avatar_get': u"Retrieve a Client Avatar",
+    u'client.avatar_set': u"Set a Client Avatar",
     u'client.cc_add': u'Add a New Credit Card',
     u'client.cc_delete': u'Delete a Credit Card',
     u'client.cc_update': u'Update a Credit Card',
@@ -37,9 +39,12 @@ METHODS = {
     u'client.contact_delete': u'Delete a Contact',
     u'client.contact_get': u'Get Contact Details',
     u'client.contact_list': u"List a Client's Contacts",
+    u'client.contact_metadata_get': u"Get a Contact's Metadata",
+    u'client.contact_metadata_single': u"Get a Contact's Metadata Value",
     u'client.contact_update': u'Update a Contact',
     u'client.count': u'Count Active Clients',
     u'client.credit_add': u'Add an Account Credit',
+    u'client.credit_apply': u"Apply a Credit to an Invoice",
     u'client.credit_comment_list': u"List a Credit's Comments",
     u'client.credit_deactivate': u'Deactivate an Account Credit',
     u'client.credit_list': u"List a Client's Credits",
@@ -79,6 +84,10 @@ METHODS = {
     u'client.service_prorate': u'Prorate a Service',
     u'client.service_update': u'Update a Service',
     u'client.set_login': u"Set a Client's Login",
+    u'client.tax_exemption_add': u"Add a new Tax Exemption",
+    u'client.tax_exemption_get': u"Get a Client's Tax Exemption",
+    u'client.tax_exemption_list': u"List a Client's Tax Exemptions",
+    u'client.tax_exemption_update': u"Update a Client's Tax Exemption",
     u'client.update': u'Update a Client',
     u'device.add': u'Add a New Device',
     u'device.comment_list': u"List a Device's Comments",
@@ -93,6 +102,7 @@ METHODS = {
     u'device.ip_assignment_delete': u'Delete a Device IP Assignment',
     u'device.ip_assignment_list': u'List Device IP Assignments',
     u'device.ip_assignment_update': u'Update a Device IP Assignment',
+    u'device.ip_block_list': u"List IP Blocks",
     u'device.ip_get_available': u'List Available IP Addresses',
     u'device.ip_get_unassigned': u'Get Unassigned IP Addresses',
     u'device.ip_group_add': u'Add a Device IP Group',
@@ -100,6 +110,7 @@ METHODS = {
     u'device.ip_group_list': u'List a Device IP Group',
     u'device.ip_group_update': u'Update a Device IP Group',
     u'device.ip_lookup': u'Look Up a Device IP',
+    u'device.ip_pool_list': u"List IP Pools",
     u'device.ip_unassign': u'Unassign a Device IP',
     u'device.list': u'List Devices',
     u'device.module_call': u'Call a Device Module Function',
@@ -146,11 +157,17 @@ METHODS = {
     u'support.ticket_post_staff_response': u'Post a Staff Response to a Ticket',
     u'support.ticket_submit': u'Submit a New Ticket',
     u'support.ticket_submit_outgoing': u'Create a New Outgoing Ticket',
+    u'support.ticket_type_list': u"Get a List of Ticket Types",
     u'support.ticket_update': u'Update a Ticket',
+    u'uber.admin_avatar_get': u"Retrieve an Admin Avatar",
+    u'uber.admin_avatar_set': u"Set an Admin Avatar",
+    u'uber.admin_get': u"User Information",
+    u'uber.admin_list': u"List User Logins",
     u'uber.api_export': u'Export Data',
     u'uber.attachment_get': u'Get an attachment',
     u'uber.attachment_list': u'List Attachments',
     u'uber.check_login': u'Verify a login and password',
+    u'uber.client_permission_list': u"List available permissions",
     u'uber.client_welcome_stats': u'Display Client Statistics',
     u'uber.comment_add': u'Add Comment',
     u'uber.comment_delete': u'Delete Comment',
@@ -159,6 +176,11 @@ METHODS = {
     u'uber.comment_update': u'Update Comment',
     u'uber.documentation': u'Download API Documentation',
     u'uber.event_list': u'Access the Event Log',
+    u'uber.file_add': u"Add a file",
+    u'uber.file_delete': u"Delete a file",
+    u'uber.file_get': u"Get a File",
+    u'uber.file_list': u"Get a List of Files",
+    u'uber.file_update': u"Update a file",
     u'uber.forgot_pass': u'Send a Password Reminder',
     u'uber.login_list': u'List User Logins',
     u'uber.mail_get': u'Get an Email From the Log',
@@ -172,6 +194,8 @@ METHODS = {
     u'uber.quick_stats_detail': u'Get Detailed System Stats',
     u'uber.service_plan_get': u'Get Service Plan Details',
     u'uber.service_plan_list': u'List Service Plans',
+    u'uber.tax_exemption_type_get': u"Get a Tax Exemption Type",
+    u'uber.tax_exemption_type_list': u"List Tax Exemption Types",
     u'uber.user_exists': u'Check whether a Client Exists',
     u'uber.username_exists': u'Check Whether a Username Exists',
 }
@@ -244,7 +268,9 @@ class _AbstractRequestHandler(object):
 
     def _encode_data(self, data):
         """URL encode data."""
-        return urlencode_unicode(data if data is not None else {})
+        data = data if data is not None else {}
+        data = to_nested_php_args(data)
+        return urlencode_unicode(data)
 
     def __getattr__(self, name):
         """If attribute accessed is a call module, return a proxy."""
